<?php
	
	require_once 'class.Data.php';
	require_once 'class.Color.php';
	require_once 'class.ColorFormat.php';

	class CtPicture extends Data {
		
		const DEFAULT_WIDTH = 128;
		const DEFAULT_HEIGHT = 64;
		
		/**
		 * A 2d pixels array. Each pixel is represented by a ColorFormat.
		 * @var $pixels
		 * @see Color
		 */
		private $pixels;
		
		/**
		 * The picture identifier (1-20).
		 * @var $id
		 */
		private $id;
		
		
		/**
		 * Format de la couleur utilise pour coder chaque pixel.
		 * @var $imageColorFormat
		 * @see ColorFormat
		 */
		private $imageColorFormat = ColorFormat::FormatMonochrome;
		
		/**
		 * The picture width in pixels. Positive integer.
		 * @var $height;
		 */
		private $height;
		
		/**
		 * 
		 * The picture height in pixels. Positive integer.
		 * @var unknown_type
		 */
		private $width;
		
		/**
		 * Constructeur. Initialise une image avec les paramtres par dfaut.
		 * @param int $id Identifiant de l'image, cd le numro (1-20)
		 * @param Color $c Couleur des pixels
		 * @see Color
		 */
		public function __construct($id = 1, $w = 128, $h = 64, $c = Color::WHITE){
			if( $id > 0 and $id < 21 ){
				$this->id = $id;
			} else {
				$this->id = 1;
			}
			
			if( is_int($w) ){
				if( $w > 0 ){
					$this->width = $w;
				} else {
					$this->width = DEFAULT_WIDTH;
				}
			} else {
				$this->width = DEFAULT_WIDTH;
			}
			
			if( is_int($h) ){
				if( $h > 0 ){
					$this->height = $h;
				} else {
					$this->height = DEFAULT_HEIGHT;
				}
			} else {
				$this->height = DEFAULT_HEIGHT;
			}
			
			$this->pixels = array();
			for( $y = 0 ; $y < $this->height ; $y++ ){
				for( $x = 0 ; $x < $this->width ; $x++ ){
					$this->pixels[] = $c;
				}
			}			
		}
		
		/**
		 * Destructor
		 */
		public function __destruct(){}
		
		/******************************************
		 * GETTERS
		 *****************************************/
		/**
		 * @return int L'identifiant de l'image.
		 */
		public function getId(){
			return $this->id;
		}
		
		/**
		 * @return int Largeur de l'image.
		 * Retourne la largeur actuelle de l'image
		 */
		public function getWidth(){
			return $this->width;
		}
		/**
		 * @return int Hauteur de l'image
		 * Renvoie la hauteur actuelle de l'image
		 */
		public function getHeight(){
			return $this->height;
		}
		
		public function setPixel( $s, $x, $y, $c = Color::BLACK ){
			if( $x > 0 and $y > 0 and $x <= $this->width and $y <= $this->height ){
				$this->pixels[$this->width * ($y - 1) + $x - 1] = $c;
			}
		}
		public function getPixel( $s, $x, $y ){
			return $this->pixels[$this->width * ($y - 1) + $x - 1];
		}
		public static function getPictureFromFile($path){
			$img = @imagecreatefromjpeg($path);
			//echo imagecolorat($img, 50, 50).'<br/>';
			if( !$img ){
				return NULL;
			}
			$width = imagesx($img);
			$height = imagesy($img);
			
			//echo "Size: ({$width},{$height})<br/>";
			
			if( $width != 128 or $height != 64 ){
				$dest_picture = imagecreatetruecolor(128, 64);
				if( !imagecopyresampled($dest_picture, $img, 0, 0, 0, 0, 128, 64, $width, $height) ){
					return NULL;
				}
				//echo 'TOTO'.'<br/>';
				$img = $dest_picture;
			}
			//echo imagecolorat($img, 50, 50).'<br/>';
			//echo imagesx($img).'<br/>';
			//echo imagesy($img).'<br/>';
//			imagejpeg($img, 'thumb.jpeg');
/*			for( $x = 0 ; $x < 128 ; $x++ ){
				for( $y = 0 ; $y < 64 ; $y++ ){
					$rgb = imagecolorat($img, $x, $y);
					$r = ($rgb >> 16) & 0xFF;
					$g = ($rgb >> 8) & 0xFF;
					$b = $rgb & 0xFF;
					imagesetpixel($img, $x, $y, ($r + $g + $b) / 3 > 0x7F ? 0xFFFFFF : 0x000000);
				}
			}
*/			
//			imagejpeg($img, 'thumb_b&w.jpeg');
			
			$pict = new Picture(Color::WHITE);
			for( $y = 0 ; $y < 64 ; $y++ ){
				for( $x = 0 ; $x < 128 ; $x++ ){
					$rgb = imagecolorat($img, $x, $y);
					$r = ($rgb >> 16) & 0xFF;
					$g = ($rgb >> 8) & 0xFF;
					$b = $rgb & 0xFF;
					//echo "{$rgb} : rgb({$r},{$g},{$b}), [".(($r + $g + $b > 381) ? Color::WHITE : Color::BLACK).']<br/>';
					$pict->setPixel(0, $x + 1, $y + 1, ($r + $g + $b > 381) ? Color::WHITE : Color::BLACK);
				}
			}
			return $pict;
		}
		
		public function toJpeg($path = NULL){
			if( !is_string($path) or $path === NULL or $path === '' ){
				echo 'No path specified.<br/>';
				return false;
			}
			// on cr une nouvelle image
			$dest = imagecreatetruecolor(
				$this->width,
				$this->height
			);
			
			imagefill($dest, 0, 0, imagecolorallocate($dest, 255, 255, 255));
			
			// black
			$color = imagecolorallocate($dest, 0, 0, 0);
			switch( $this->imageColorFormat ){
				case ColorFormat::FormatMonochrome :
					for( $x = 0 ; $x < $this->width ; $x++ ){
						for( $y = 0 ; $y < $this->height ; $y++ ){
							if( $this->getPixel(0, $x + 1, $y + 1) == Color::BLACK ){
								imagesetpixel($dest, $x, $y, $color);
							}
						}
					}
					break;
					// G65
					// White - Orange - Green - Blue (as Black)
				case ColorFormat::Format4Colors :
					break;
				case ColorFormat::Format8_Colors :
					// true colors (RGB16)
				default :
					break;
					
			}
			
			$ext = substr($path, ($r = strrpos($path, '.')) ? $r : 0);
			switch( $ext ){
				case '.png' :
					// 0 = pas de compression / no compression
					imagepng($dest, $path, 0, PNG_NO_FILTER);
				break;
				case '.xbm' :
					imagexbm($dest, $path, imagecolorallocate($dest, 255, 255, 255));
				break;
					case '.gif' :
					imagegif($dest, $path);
				case '.wbmp' :
					imagewbmp($dest, $path, imagecolorallocate($dest, 255, 255, 255));
				break;
				case '.bmp' :
					imagebmp($dest, $path);
				break;
				case '.jpeg' :
				case '.jpg' :
				default :
					// [^\.]+ \.[^\.]*
					if( $ext == '.' ){
						$path .= 'jpg';
					}
					if( stripos($ext, '.') === false ){
						$path .= '.jpg';
					}
					$path = 'Picture_'.$this->id.'.jpg';
					imagejpeg($dest, $path, 100);
				break;
			}
			
			// enregistre dans le fichier $path avec la meilleure qualit
			imagedestroy($dest);			
		}
		
		public function getPictureAs($fileType = 1){
			// on cr une nouvelle image
			$dest = imagecreatetruecolor(
				$this->width,
				$this->height
			);
			
			// set background color to white
			imagefill($dest, 0, 0, imagecolorallocate($dest, 255, 255, 255));
			
			// black
			$color = imagecolorallocate($dest, 0, 0, 0);
			switch( $this->imageColorFormat ){
				case ColorFormat::FormatMonochrome :
				default:
					for( $x = 0 ; $x < $this->width ; $x++ ){
						for( $y = 0 ; $y < $this->height ; $y++ ){
							if( $this->getPixel(0, $x + 1, $y + 1) == Color::BLACK ){
								imagesetpixel($dest, $x, $y, $color);
							}
						}
					}
				break;
			}
			
			ob_start();
			imagegif($dest);
			$data = ob_get_contents();
			ob_end_clean();
			return $data;
		}
		
		public function __clone(){
		}
	}

?>