<?php

	require_once('../Data/class.Project.php');
	
	class FileWriter {
		var $targetModel;
		
		const G1M = 1;
		const G2M = 2;
		
		public function __construct($tM = self::G1M){
			$this->targetModel = $tM;
		}
		public function writeProject($project){
			if( !($project instanceof Project) ){
				return false;
			}
			
			$fileName = $project->getName();
			for( $i = 0 ; $i < strlen($fileName) ; $i++ ){
				if( !($fileName[$i] >= 'A' and $fileName[$i] <= 'Z' or
					$fileName[$i] >= 'a' and $fileName[$i] <= 'z' or
					$fileName[$i] >= '0' and $fileName[$i] <= '9' or
					in_array($fileName[$i], array('+', '-', '*', '/', '.', '{', '}', '[', ']', ' ', '"', '~', "'")) ) ){
					$fileName[$i] = ' ';
				}
			}
			$fileName = rtrim($fileName, ' ');
			$fileName = substr($fileName, 0, 8);

			switch( $this->targetModel ){
				case FileWriter::G1M:
					$fileName .= '.g1m';
					break;
				default:
					$fileName = '.txt';
			}
			
			$fileName = 'generated.g1m';
			
			$programs = $project->getPrograms();
			$matrix = $project->getMatrix();
			$pictures = $project->getPictures();
			$lists = $project->getLists();
			
			$handle = fopen($fileName, 'wb');
			if( $handle === false ){
				return false;
			}
			$dataToWrite = '';
			foreach( $programs as $prog ){
				$dataToWrite .= $this->getProgramData($prog);
			}
			foreach( $matrix as $m ){
				$dataToWrite .= $this->getMatrixData($m);
			}
			foreach( $pictures as $picture ){
				$dataToWrite .= $this->getPictureData($picture);
			}
			foreach( $lists as $list ){
				$dataToWrite .= $this->getListData($list);
			}
			$this->writeHeader($project, $handle, strlen($dataToWrite) + 32, count($project->getAll()));
			fwrite($handle, $dataToWrite);
			fclose($handle);
		}
		
		private function getMatrixData($matrix){
			$data;
			if( $matrix->getId() != 'Ans' ){
				$data = "MAT ".$matrix->getId()."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01main\x00\x00\x00\x00MAT_".$matrix->getId()."\x00\x00\x00\x06\x00\x00\x00";
			} else {
				$data = "MAT Ans\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01main\x00\x00\x00\x00MAT_Ans\x00\x06\x00\x00\x00";
			}
			$dataSize = 16 + $matrix->getRows() * $matrix->getColumns() * 12;
			$data .= sprintf(
				"%c%c%c%c",
				($dataSize % 256),
				($dataSize >> 8),
				($dataSize >> 16),
				($dataSize >> 24)
			);
			// 8 x 0x00
			$data .= "\x00\x00\x00\x00\x00\x00\x00\x00";
			// rows x columns (2 bytes each)
			$data .= sprintf("%c%c", ($matrix->getRows() >> 8),($matrix->getRows() % 256));
			$data .= sprintf("%c%c", ($matrix->getColumns() >> 8),($matrix->getColumns() % 256));
			// 4 x 0x00
			$data .= "\x00\x00\x00\x00";
			// cells
			for( $r = 0 ; $r < $matrix->getRows() ; $r++ ){
				for( $c = 0 ; $c < $matrix->getColumns() ; $c++ ){
					// 12 bytes each cell with non complexes numbers
					$data .= $matrix->getCell($r + 1, $c + 1)->__toString();
				}
			}
			
			return $data;
			
		}
		
		private function getFileListData($file){
			$data = "LISTFILE ".$file->getId();
			if( $file->getId() < 10 ){
				$data .= "\x00";
			}
			$data .= "\x00\x00\x00\x00\x00\x00\x00\x00\x01main\x00\x00\x00\x001LIST".$file->getId();
			if( $file->getId() < 10 ){
				$data .= "\x00";
			}
			$data .= "\x00\x05\x00\x00";
			return $data;
		}
		
		private function getListData($list){
			$data = "LIST ".$list->getId();
			for( $i = 0 ; $i < 3 - strlen($list->getId()) ; $i++ ){
				$data .= "\x00";
			}
			$data .= "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01main\x00\x00\x00\x00".$list->getFileId()."LIST".$list->getId();
			if( $list->getId() < 10 ){
				$data .= "\x00";
			}
			$data .= "\x00\x05\x00\x00";
			//
			$dataSize = 16 + 12 * $list->getSize();
			$data .= sprintf("%c", (($dataSize >> 8) & 0xFF));
			$data .= sprintf("%c", ($dataSize & 0xFF));
			$data .= "\x00\x00\x00\x000\x00\x02\x00\x00uN";
			$data .= sprintf("%c", (($list->getSize() >> 8) & 0xFF));
			$data .= sprintf("%c", ($list->getSize() & 0xFF));
			$data .= "\xa5\xf8\x00\x00\x00\x00";
			
			for( $i = 0 ; $i < $list->getSize() ; $i++ ){
				$data .= $list->getRow($i + 1)->__toString();
			}
			
			return $data;
		}
		
		private function getPictureData($picture){
//			echo 'toto';
			$data = "PICTURE ".$picture->getId()."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01main\x00\x00\x00\x00PICT".$picture->getId()."\x00\x00\x00\x07\x00\x00\x08\x00\x00\x00\x00";
			for( $y = 1 ; $y < 65 ; $y++ ){
				for( $x = 1 ; $x < 129 ; $x += 8 ){
					$val = 0;
					for( $b = 0 ; $b < 8 ; $b++ ){
						$val = $val * 2 + (($picture->getPixel(0, $x + $b, $y) == Color::BLACK) ? 1 : 0);
					}
					$data .= sprintf("%c", $val);
//					echo $val.'<br/>';
				}
			}
			for( $i = 0 ; $i < 1024 ; $i++ ){
				$data .= "\x00";
			}
			return $data;
		}
		
		/**
		 * @french
		 * Cette fonction renvoie un programme sous la forme d'une chaine G1m
		 * @endfrench
		 * @english
		 * This function return the program under a g1m string.
		 * @endenglish
		 * @param unknown_type $program
		 */
		private function getProgramData($program){
			$data = "PROGRAM\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01system\x00\x00";
			
			// get the 8 first characters
			$programName = substr($program->getName(), 0, (strlen($program->getName()) > 8 ? 8 : strlen($program->getName())));
			for( $i = 0 ; $i < strlen($programName) ; $i++ ){
				// allowed charachters in the program name
				// others characters can be included but i have not an exhaustive list now
				// so i'll put add them later
				// rho & theta can also be added but don't know their code
				if( !($programName[$i] >= 'A' and $programName[$i] <= 'Z' or
					$programName[$i] >= 'a' and $programName[$i] <= 'z' or
					$programName[$i] >= '0' and $programName[$i] <= '9' or
					in_array($programName[$i], array('+', '-', '*', '/', '.', '{', '}', '[', ']', ' ', '"', '~', "'")) ) ){
					$programName[$i] = ' ';
				}
			}
			// after filter : [TOTO???] => [TOTO   ]
			// remove useless space at the end of the name
			// spaces replaces bad characters
			$programName = rtrim($programName, ' ');
			
			$programCode = $program->getCode();
			$length = strlen($programCode);
			if( $length < 2 ){
				while( $length < 2 ){
					$programCode .= "\x00"; $length++;
				}
			} else {
				$programCode .= "\x00\x00";
				$length += 2;
				while( ($length + 2) % 4 !== 0 ){
					$programCode .= "\x00";
					$length++;
				}
			}
			
			$data .= $programName;
			for( $i = 0 ; $i < 8 - strlen($programName) ; $i++ ){
				$data .= "\x00";
			}
			$data .= "\x01\x00\x00";
			// header size + code size
			$programSize = 54 + strlen($programCode);
			// size of the block must be a multiple of 4
			if( $programSize % 4 !== 0 ){
				// padding with 0x00
				for( $i = 0 ; $i < 4 - $programSize % 4 ; $i++ ){
					$programCode .= "\x00";
				}
			}
			$programSize = strlen($programCode) + 10;
			$data .= sprintf("%c", $programSize >> 8);
			$data .= sprintf("%c", $programSize % 256);
			
			$data .= "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";
			if( $program->isBase() ){
				$data .= "\x01";
			} else {
				$data .= "\x00";
			}
			$data .= "\x00";
			
			$data .= $programCode;
			return $data;
		}
		
		private function writeHeader($project, $h, $fileSize, $elements){
			switch( $this->targetModel ){
				case self::G1M:
				case self::G2M:
					fwrite($h, "\xaa\xac\xbd\xaf\x90\x88\x9a\x8d\xce\xff\xef\xff\xef\xff");
					$fileSizeCheck = 190 - $fileSize % 256;
					if( $fileSizeCheck < 0 ){
						$fileSizeCheck += 256;
					}
					fprintf($h, "%c", $fileSizeCheck);
					fwrite($h, "\xfe");
					$fileSizeCheck = 0xFFFFFFFF - $fileSize;
					fprintf(
						$h,
						"%c%c%c%c",
						$fileSizeCheck >> 24,
						$fileSizeCheck >> 16,
						$fileSizeCheck >> 8,
						$fileSizeCheck % 256
					);
					$secondCheck = 71 - $fileSize % 256;
					if( $secondCheck < 0 ){
						$secondCheck += 256;
					}
					fprintf($h, "%c", $secondCheck);
					if( $this->targetModel == FileWriter::G1M ){
						fwrite($h, "\x00\x00\x00\x00\x00\x00\x00\x00\x00");
					} else {
						// don't know what this byte is used for
						fwrite($h, "\x01\x00\x00\x00\x00\x00\x00\x00\x00");
					}
					fprintf($h, "%c", (0xff - ($elements >> 8)));
					fprintf($h, "%c", (0xff - $elements % 256));
			}
		}
	}
	
	
	/*
		To fix:
			- Labels with Theta & Rho
	*/
	function textToCat($code){
		$code = str_replace(array(
				'Lbl ',
				'Mat ',
				'List ',
				'If ',
				'Then ',
				'IfEnd',
				'Frac ',
				'ClrText',
				'Locate ',
				'For ',
				'To ',
				'Next',
				'Sqrt('
			),
			array(
				'\\Lbl ',
				'\\Mat ',
				'\\List ',
				'\\If ',
				'\\Then ',
				'\\IfEnd',
				'\\Frac ',
				'\\ClrText',
				'\\Locate ',
				'\\For ',
				'\\To ',
				'\\Next',
				'\\sqrt'
			), $code);

		$code = preg_replace('/Goto ([A-Z])/s', "\\Goto$1", $code);
		return $code;
	}	

?>