<?php
	include_once('class.Project.php');
	
	/**
		Class used to write Fxi files. FXI are FX-Interface files format with FXB (backup) & FXP (pro).
		FXI is a binary file format.
	*/
	
	class FxiWriter {
		private static $table = array(
			0xe7, 0xe6, 0xe5, 0xe4, 0xe3, 0xe2, 0xe1, 0xe0, 0xef, 0xee, 0xed, 0xec, 0xeb, 0xea, 0xe9, 0xe8, 
			0xf7, 0xf6, 0xf5, 0xf4, 0xf3, 0xf2, 0xf1, 0xf0, 0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8, 
			0xc7, 0xc6, 0xc5, 0xc4, 0xc3, 0xc2, 0xc1, 0xc0, 0xcf, 0xce, 0xcd, 0xcc, 0xcb, 0xca, 0xc9, 0xc8, 
			0xd7, 0xd6, 0xd5, 0xd4, 0xd3, 0xd2, 0xd1, 0xd0, 0xdf, 0xde, 0xdd, 0xdc, 0xdb, 0xda, 0xd9, 0xd8, 
			0xa7, 0xa6, 0xa5, 0xa4, 0xa3, 0xa2, 0xa1, 0xa0, 0xaf, 0xae, 0xad, 0xac, 0xab, 0xaa, 0xa9, 0xa8, 
			0xb7, 0xb6, 0xb5, 0xb4, 0xb3, 0xb2, 0xb1, 0xb0, 0xbf, 0xbe, 0xbd, 0xbc, 0xbb, 0xba, 0xb9, 0xb8, 
			0x87, 0x86, 0x85, 0x84, 0x83, 0x82, 0x81, 0x80, 0x8f, 0x8e, 0x8d, 0x8c, 0x8b, 0x8a, 0x89, 0x88, 
			0x97, 0x96, 0x95, 0x94, 0x93, 0x92, 0x91, 0x90, 0x9f, 0x9e, 0x9d, 0x9c, 0x9b, 0x9a, 0x99, 0x98, 
			0x67, 0x66, 0x65, 0x64, 0x63, 0x62, 0x61, 0x60, 0x6f, 0x6e, 0x6d, 0x6c, 0x6b, 0x6a, 0x69, 0x68, 
			0x77, 0x76, 0x75, 0x74, 0x73, 0x72, 0x71, 0x70, 0x7f, 0x7e, 0x7d, 0x7c, 0x7b, 0x7a, 0x79, 0x78, 
			0x47, 0x46, 0x45, 0x44, 0x43, 0x42, 0x41, 0x40, 0x4f, 0x4e, 0x4d, 0x4c, 0x4b, 0x4a, 0x49, 0x48, 
			0x57, 0x56, 0x55, 0x54, 0x53, 0x52, 0x51, 0x50, 0x5f, 0x5e, 0x5d, 0x5c, 0x5b, 0x5a, 0x59, 0x58, 
			0x27, 0x26, 0x25, 0x24, 0x23, 0x22, 0x21, 0x20, 0x2f, 0x2e, 0x2d, 0x2c, 0x2b, 0x2a, 0x29, 0x28, 
			0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x30, 0x3f, 0x3e, 0x3d, 0x3c, 0x3b, 0x3a, 0x39, 0x38, 
			0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00, 0x0f, 0x0e, 0x0d, 0x0c, 0x0b, 0x0a, 0x09, 0x08, 
			0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 0x10, 0x1f, 0x1e, 0x1d, 0x1c, 0x1b, 0x1a, 0x19, 0x18
		);
		
		public static $AFX		= 1;
		public static $CFX		= 2;
		
		public static $G100_PLUS = 10;
		public static $G100		= 11;
		public static $G80		= 12;
		public static $G65		= 13;
		public static $G35_PLUS	= 14;
		public static $G35		= 15;
		public static $G25		= 16;
		
		// private variables
		// AFX or CFX
		private $targetModelType;
		private $targetModel;
		private $generalComment;
		private $programComment;
		private $transfertDataComment;
		private $hardCopyComment;
		
		/**
			Constructor
			@param $target Model type : AFX or CFX
		*/
		public function __construct(){//$targetModel = self::$AFX, $targetCalc = self::$G100_PLUS){
			$this->targetModelType		= $targetModel;
			$this->targetModel			= $targetCalc;
			
			$this->generalComment		= '';
			$this->programComment		= '';
			$this->transfertDataComment	= '';
			$this->hardCopyComment		= '';
		}
		
		/**
			Specific functions to Fxi files
			General comment
			Program comment
			TransfertData comment
			HardCopy comment
			
			80 characters max
		*/
		public function setGeneralComment($comment){
			if( is_string($comment) ){
				$this->generalComment = substr($comment, 0, 80);
			}
		}
		public function setProgramComment($comment){
			if( is_string($comment) ){
				$this->programComment = substr($comment, 0, 80);
			}
		}
		public function setTransfertDataComment($comment){
			if( is_string($comment) ){
				$this->transfertDataComment = substr($comment, 0, 80);
			}
		}
		public function setHardCopyComment($comment){
			if( is_string($comment) ){
				$this->hardCopyComment = substr($comment, 0, 80);
			}
		}
		
		public static function writeProject($project){
			if( !($project instanceof Project ) ){
				return false;
			}
			
			$this->writeHeader();
			// program(s) must be written first
			foreach( $programs as $program ){
				$this->writeProgram($program);
			}
			foreach( $nonProgram as $data ){
				if( $data instanceof Picture ){
					$this->writePicture($data);
				} else {
					// not supported
					// to add
				}
			}
		}
		
		private function writeHeader(){
			$str = '';
			
			if( $this->targetModelType == FxiWriter::AFX ){
				$str .= ("\xd4\xd7\x1f"
						.'FX-INTERFACE - YELLOW COMPUTING'
						// 40 x 0x00
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x07\x00"
						// 80 x 0x00
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x07\x00\xff\xff\x00\x00\x10\x00"
						.'CYCFXListItemMCS'
						."\x00\x00\x00\x00\x00\x00\x00\x00,"
						."\x03\x00\x15\x18\x00\x00\x00\x00");
			} else {
				$str .= ("\xd5\xd7\x1f"
						.'FX-INTERFACE - YELLOW COMPUTING'
						// 40 x 0x00
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x06\x00"
						// 80 x 0x00
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x07\x00\xff\xff\x00\x00\x0f\x00"
						.'CYCFXListItemNP'
						."\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x03\x00\x15\x18\x00\x00\x00\x00");
			}
			
			$str .= sprintf("%c%s", strlen($this->generalComment), $this->generalComment);
			
			switch( $this->targetModel ){
				case FxiWriter::G100_PLUS :
				case  FxiWriter::G100 : 
					$str .= "\x52";
					break;
				case  FxiWriter::G80 : 
					$str .= "\x46";
					break;
				case  FxiWriter::G35_PLUS :
				case  FxiWriter::G35 : 
					$str .= "\x55";
					break;
				default : $str .= "\x00";
			}
			
			$str .=	("\xa0"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x01\x00\x00\x00\x01\x00\x03\x00\x55\x1a\x00\x00"
					."\x04\x00");

			$str .= sprintf("%c%s", strlen($this->programComment), $this->programComment);

			$str .=	("\x5b\x80"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x08\x13\x00\x00"
					."\x22\x00\x00\x83\x80"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x03\x00\x00\x00\x01\x00\x03\x00\x55\x1a\x00\x00"
					."\x20\x00");

			$str .= sprintf("%c%s", strlen($this->transfertDataComment), $this->transfertDataComment);

			$str .=	("\x1f\x81"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x04\x00\x00\x00\x02\x00\x00\x00\x08\x13\x00\x00"
					."\x21\x00\x00\x83\x80"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x05\x00\x00\x00\x00\x00\x02\x00\x58\x10\x00\x00"
					."\x1e\x00");

			$str .= sprintf("%c%s", strlen($this->hardCopyComment), $this->hardCopyComment);

			$str .=	("\xcc\x80"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x01\x80\x00\x00\x06\x00\x00\x00\x01\x00\x00\x00\x08\x11\x00\x00"
					."\x1e\x00\x00\x83\x80"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00");

			return $str;
		}
		
		private function writeProgram($prog){
			$str = "0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x01, 0x00, 0xba, 0x1e, 0x00, 0x00, 0x22, 0x00";

			$str .= sprintf("%c%s", strlen($prog->getName()), $prog->getName());
			$str .= "\x83\x80";

			// 42 x 0x00
			$str .= ("\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
					."\x00\x00");
			
			$str .= sprintf("%s", $prog->getName());
			for( $i = 0 ; $i < 24 + 8 - strlen($prog->getName()) ; $i++ ){
				$str .= "\x00";
			}

			$str .= "\x01\x00\x01\x00";

			$str .= sprintf("%c", ((strlen($prog->getCode()) + 49) & 0xFF));
			$str .= sprintf("%c", ((strlen($prog->getCode()) + 49) >> 8));

			if( $this->targetModel == FxiWriter::CFX ){
				$str .= "TXT";
				$str .= "\x00";
				$str .= "PG";
				$str .= sprintf("%c", ((strlen($prog->getCode()) + 3) >> 24) & 0xFF);
				$str .= sprintf("%c", ((strlen($prog->getCode()) + 3) >> 16) & 0xFF);
				$str .= sprintf("%c", ((strlen($prog->getCode()) + 3) >>  8) & 0xFF);
				$str .= sprintf("%c", ((strlen($prog->getCode()) + 3) & 0xFF));
			}

			// program name
			$str .= $prog->getName();

			// fill with 0xff
			for( $i = 0 ; $i < 16 - strlen($prog->getName()) ; $i++ ){
				$str .= "\xff";
			}

			// password
			if( $prog->hasPassword() ){
				$str .= $prog->getPassword();
			}
			
			// fill with 0xff
			for( $i = 0 ; $i < 8 - strlen($prog->getPassword()) ; $i++ ){
				$str .= "\xff";
			}

			if( $prog->isBaseProgram() ){
				$str .= "BN";
			} else {
				$str .= "NL";
			}

			// 12 x 0xff
			$str .= "\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff";

			// code
			$str .= $prog->getCode();
			$str .= "\xff";
		}
		
		private function writePicture($pict){
			// to do : add afx version & remove 'true'
			// manage picture 10-20
			if( $this->targetModel == FxiWriter::CFX || true){
				$str .= "\x01\x80\x00\x00\x00\x00\x00\x00\x02\x00\x01\x00\x3a\x1c\x00\x00\x21\x00";
				// size of the picture name "Picture<n>" number is between 1 & 6 so length is always 8
				$str .= "\x08";
				// the picture name
				$str .= "Picture";
				// 1,2,3,4,5,6
				$str .= $pict->getId();
				$str .= "\x83\x80";

				// 42 x 0x00
				$str .= ("\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00");

				// the picture name
				$str .= "Picture";
				// 1,2,3,4,5,6
				$str .= $pict->getId();

				// 24 x 0x00
				$str .= ("\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
						."\x00\x00\x00\x00");
						
				$str .= ("\x01\x00\x01\x00\x40\x10"."IMG"."\x00"."PC"."\x00\x40\x00\x80"."Picture");
				// 1,2,3,4,5,6
				$str .= $pict->getId();

				$str .= "\xff\xff\xff\xff\xff\xff\xff\xff";

				$str .= ("DRUWF"."\x00\x04\x00\x01");

				// 13 x 0xff
				$str .= ("\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff"
						."\xff\xff\xff");

				$byte = 0;
				for( $sheet = 0x00 ; $sheet < 0x04 ; $sheet += 0x01 ){
					$str .= "\x00\x01\x00";
					$str .= chr($sheet);

					for( $w = 128 ; $w > 0 ; $w -= 8 ){
						for( $h = 64 ; $h > 0 ; $h-- ){
							for( $c = 0 ; $c < 8 ; $c++ ){
								if( $pict->getPixel($w - 8 + $c, $h - 1) ){
									$byte = 2 * $byte + 1;
								} else {
									$byte = 2 * $byte;
								}
							}
							$str .= chr($byte);
							$byte = 0;
						}
					}
				}
			}
			
			return $str;
		}
		
		/**
			crypt or decrypt FXI files with the conversion table
		*/
		public static function deCrypt(&$bytes){
			for( $i = 0 ; $i < strlen($bytes) ; $i++ ){
				$bytes[$i] = chr(self::$table[ord($bytes[$i])]);
			}
		}		
	}

?>